@@ -190,18 +190,18 @@ class ProcessExecutionErrorTest(test_base.BaseTestCase):
         stdout = """
         Lo, praise of the prowess of people-kings
         of spear-armed Danes, in days long sped,
-        we have heard, and what honot the athelings won!
+        we have heard, and what honor the athelings won!
         Oft Scyld the Scefing from squadroned foes,
         from many a tribe, the mead-bench tore,
-        awing the earls. Since erse he lay
+        awing the earls. Since erst he lay
         friendless, a foundling, fate repaid him:
-        for he waxed under welkin, in wealth he trove,
+        for he waxed under welkin, in wealth he throve,
         till before him the folk, both far and near,
         who house by the whale-path, heard his mandate,
-        gabe him gits: a good king he!
+        gave him gifts: a good king he!
         To him an heir was afterward born,
         a son in his halls, whom heaven sent
-        to favor the fol, feeling their woe
+        to favor the folk, feeling their woe
         that erst they had lacked an earl for leader
         so long a while; the Lord endowed him,
         the Wielder of Wonder, with world's renown.
@@ -758,7 +758,8 @@ class SshExecuteTestCase(test_base.BaseTestCase):
         fake_stdout.channel.recv_exit_status.return_value = rc
         fake_stdout.read.return_value = b'password="secret"'
 
-        fake_stderr = six.BytesIO(b'password="foobar"')
+        fake_stderr = mock.Mock()
+        fake_stderr.read.return_value = b'password="foobar"'
 
         command = 'ls --password="bar"'
 
@@ -778,6 +779,20 @@ class SshExecuteTestCase(test_base.BaseTestCase):
             self.assertEqual('ls --password="***"', err.cmd)
             self.assertNotIn('secret', str(err))
             self.assertNotIn('foobar', str(err))
+
+            # test ssh_execute with sanitize_stdout=False
+            err = self.assertRaises(processutils.ProcessExecutionError,
+                                    processutils.ssh_execute,
+                                    connection, command,
+                                    check_exit_code=check,
+                                    sanitize_stdout=False)
+
+            self.assertEqual(rc, err.exit_code)
+            self.assertEqual('password="***"', err.stdout)
+            self.assertEqual('password="***"', err.stderr)
+            self.assertEqual('ls --password="***"', err.cmd)
+            self.assertNotIn('secret', str(err))
+            self.assertNotIn('foobar', str(err))
         else:
             o, e = processutils.ssh_execute(connection, command,
                                             check_exit_code=check)
@@ -786,6 +801,15 @@ class SshExecuteTestCase(test_base.BaseTestCase):
             self.assertIn('password="***"', fixture.output)
             self.assertNotIn('bar', fixture.output)
 
+            # test ssh_execute with sanitize_stdout=False
+            o, e = processutils.ssh_execute(connection, command,
+                                            check_exit_code=check,
+                                            sanitize_stdout=False)
+            self.assertEqual('password="secret"', o)
+            self.assertEqual('password="***"', e)
+            self.assertIn('password="***"', fixture.output)
+            self.assertNotIn('bar', fixture.output)
+
     def test_compromising_ssh1(self):
         self._test_compromising_ssh(rc=-1, check=True)
 
@@ -968,3 +992,16 @@ class PrlimitTestCase(test_base.BaseTestCase):
             stderr=mock.ANY, close_fds=mock.ANY,
             preexec_fn=mock.ANY, shell=mock.ANY,
             cwd=mock.ANY, env=mock.ANY)
+
+    @mock.patch.object(processutils.subprocess, 'Popen')
+    def test_python_exec(self, sub_mock):
+        mock_subprocess = mock.MagicMock()
+        mock_subprocess.communicate.return_value = (b'', b'')
+        sub_mock.return_value = mock_subprocess
+        args = ['/a/command']
+        prlimit = self.limit_address_space()
+
+        processutils.execute(*args, prlimit=prlimit, check_exit_code=False,
+                             python_exec='/fake_path')
+        python_path = sub_mock.mock_calls[0][1][0][0]
+        self.assertEqual('/fake_path', python_path)
